from django.contrib.auth.models import BaseUserManager


class AuthManager(BaseUserManager):

    def create_user(self, email, password=None, **other_fields):
        user = self.model(email=self.normalize_email(email), **other_fields)
        user.set_password(password)
        user.save()
        return user

    def create_superuser(self, email, password, **other_fields):
        other_fields = other_fields or {}
        other_fields['is_superuser'] = True
        return self.create_user(email, password, **other_fields)



from ..utils import get_setting


def test_get_setting_with_primitive():
    setting = 'abc'

    result = get_setting(setting, None)

    assert result == setting


def test_get_setting_with_lambda():
    setting = lambda request: request
    request = object()

    result = get_setting(setting, request)

    assert result == request



from .test import *  # noqa

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'test',
        'USER': os.environ.get('PG_USER'),
        'PASSWORD': os.environ.get('PG_PASSWORD'),
        'HOST': '127.0.0.1',
        'PORT': 5434,
    }
}



from unittest.mock import patch

from django.conf import settings
from django.test import TestCase
import pytest
import responses

from . import mocks
from ... import http
from ...backends import KagisoBackend
from ...exceptions import AuthAPIUnexpectedStatusCode, EmailNotConfirmedError
from ...models import KagisoUser


class KagisoBackendTest(TestCase):

    @responses.activate
    @patch('kagiso_auth.backends.KagisoUser', autospec=True)
    def test_authenticate_valid_credentials_returns_user(self, MockKagisoUser):  # noqa
        email = 'test@email.com'
        password = 'random'
        user = KagisoUser(email=email, password=password)

        MockKagisoUser.sync_user_data_locally.return_value = user

        url, _ = mocks.post_sessions(
            http.HTTP_200_OK,
            last_sign_in_via=settings.APP_NAME
        )

        backend = KagisoBackend()
        result = backend.authenticate(
            email=email,
            password=password,
        )

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == url

        assert isinstance(result, KagisoUser)
        assert result.id == user.id

    @responses.activate
    def test_authenticate_valid_credentials_creates_local_user_if_none(self):
        email = 'test@email.com'
        password = 'random'

        data = {
            'id': 55,
            'email': email,
            'first_name': 'Fred',
            'last_name': 'Smith',
            'is_staff': True,
            'is_superuser': True,
            'profile': {'age': 40, },
            'created_via': settings.APP_NAME,
            'last_sign_in_via': settings.APP_NAME
        }

        session_url, data = mocks.post_sessions(
            http.HTTP_200_OK,
            **data
        )

        backend = KagisoBackend()
        result = backend.authenticate(
            email=email,
            password=password,
        )

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == session_url

        assert result.email == data['email']

    @responses.activate
    def test_authenticate_invalid_status_code_raises(self):
        email = 'test@email.com'
        password = 'random'
        url, api_data = mocks.post_users(1, email)
        KagisoUser.objects.create_user(email, password)

        mocks.post_sessions(500)

        backend = KagisoBackend()

        with pytest.raises(AuthAPIUnexpectedStatusCode):
            backend.authenticate(
                email=email,
                password=password,
            )

    @responses.activate
    @patch('kagiso_auth.backends.KagisoUser', autospec=True)
    def test_authenticate_with_social_sign_in_returns_user(self, MockKagisoUser):  # noqa
        email = 'test@email.com'
        password = 'secret'
        strategy = 'facebook'
        user = KagisoUser(email=email, password=password)

        MockKagisoUser.sync_user_data_locally.return_value = user

        url, data = mocks.post_sessions(http.HTTP_200_OK)
        mocks.put_users(1, email)

        backend = KagisoBackend()
        result = backend.authenticate(
            email=email,
            strategy=strategy,
        )

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == url

        assert isinstance(result, KagisoUser)
        assert result.id == user.id

    @responses.activate
    def test_authenticate_invalid_credentials_returns_none(self):
        email = 'test@email.com'
        password = 'incorrect'
        mocks.post_users(1, email)
        KagisoUser.objects.create_user(email, password)
        url, data = mocks.post_sessions(http.HTTP_404_NOT_FOUND)

        backend = KagisoBackend()
        result = backend.authenticate(email=email, password=password)

        assert len(responses.calls) == 2
        assert responses.calls[1].request.url == url

        assert not result

    @responses.activate
    def test_authenticate_unconfirmed_email_raises(self):
        email = 'test@email.com'
        password = 'random'
        url, api_data = mocks.post_users(1, email)
        KagisoUser.objects.create_user(email, password)

        mocks.post_sessions(http.HTTP_422_UNPROCESSABLE_ENTITY)

        backend = KagisoBackend()

        with pytest.raises(EmailNotConfirmedError):
            backend.authenticate(email=email, password=password)



from datetime import datetime

from django import forms
from django.forms.extras import widgets
from django.utils import timezone


from .models import KagisoUser


def validate_passwords_match(form):
    password = form.cleaned_data.get('password')
    confirm_password = form.cleaned_data.get('confirm_password')

    if password and password != confirm_password:
        message = 'Passwords do not match'
        form.add_error('confirm_password', message)


class SignInForm(forms.Form):
    email = forms.EmailField(label='Email Address')
    password = forms.CharField(widget=forms.PasswordInput())
    remember_me = forms.BooleanField(required=False)


class SignUpForm(forms.Form):
    GENDER_CHOICES = (('MALE', 'Male'), ('FEMALE', 'Female'),)
    REGION_CHOICES = (
        ('', 'Select Region'),
        ('EASTERN_CAPE', 'Eastern Cape'),
        ('FREE_STATE', 'Free State'),
        ('GAUTENG', 'Gauteng'),
        ('KWAZULU_NATAL', 'Kwazulu Natal'),
        ('LIMPOPO', 'Limpopo'),
        ('MPUMALANGA', 'Mpumalanga'),
        ('NORTH_WEST', 'North West'),
        ('NORTHERN_CAPE', 'Northern Cape'),
        ('WESTERN_CAPE', 'Western Cape'),
    )
    ALERT_CHOICES = (('EMAIL', 'Email'), ('SMS', 'SMS'))
    MOBILE_REGEX = r'^\d{10}$'

    # --- Form fields ---
    email = forms.EmailField(
        widget=forms.TextInput(
            attrs={'placeholder': 'eg. name@emailaddress.com'}
        ),
        error_messages={'required': 'Please enter a valid email address'}
    )
    first_name = forms.CharField(
        widget=forms.TextInput(attrs={'placeholder': 'eg. John'}),
        error_messages={'required': 'Please enter your name'}
    )
    last_name = forms.CharField(
        widget=forms.TextInput(attrs={'placeholder': 'eg. Doe'}),
        error_messages={'required': 'Please enter your last name'}
    )
    password = forms.CharField(widget=forms.PasswordInput(), min_length=8)
    confirm_password = forms.CharField(widget=forms.PasswordInput())
    mobile = forms.RegexField(
        regex=MOBILE_REGEX,
        widget=forms.TextInput(attrs={
            'placeholder': 'eg. 0821111111',
            'pattern': MOBILE_REGEX,
            'title': 'eg. 0821111111'}),
        error_messages={
            'required': 'Correct mobile number format: 0821111111'
        },
        error_message=('Correct mobile number format: 0821111111')
    )
    gender = forms.ChoiceField(
        error_messages={'required': 'Please select a gender'},
        choices=GENDER_CHOICES,
        widget=forms.RadioSelect
    )
    region = forms.ChoiceField(
        error_messages={'required': 'Please select a region'},
        choices=REGION_CHOICES
    )
    birth_date = forms.DateField(
        label='Birthday',
        widget=widgets.SelectDateWidget(
            months={
                0: 'Select Month',
                1: 'January',
                2: 'February',
                3: 'March',
                4: 'April',
                5: 'May',
                6: 'June',
                7: 'July',
                8: 'August',
                9: 'September',
                10: 'October',
                11: 'November',
                12: 'December',
            },
            years=['Select Year'] +
            list(range(datetime.now().year, 1900, -1))
        )
    )
    alerts = forms.MultipleChoiceField(
        label='How would you like to receive alerts?',
        choices=ALERT_CHOICES,
        widget=forms.CheckboxSelectMultiple,
        required=False
    )

    # --- Instance variables ---
    social_sign_in = False

    @classmethod
    def create(cls, post_data=None, oauth_data=None):
        form = cls(post_data, initial=oauth_data)

        # Social sign ups don't require passwords...
        if oauth_data:
            form.social_sign_in = True
            form._remove_password_fields()

        return form

    def clean(self):
        validate_passwords_match(self)
        return self.cleaned_data

    def save(self, app_name):
        user = KagisoUser()
        user.email = self.cleaned_data['email']

        if self.social_sign_in:
            user.email_confirmed = timezone.now()
        else:
            user.set_password(self.cleaned_data['password'])

        user.first_name = self.cleaned_data['first_name']
        user.last_name = self.cleaned_data['last_name']
        user.profile = {
            'mobile': self.cleaned_data['mobile'],
            'gender': self.cleaned_data['gender'],
            'region': self.cleaned_data['region'],
            'birth_date': str(self.cleaned_data['birth_date']),
            'alerts': self.cleaned_data['alerts'],
        }
        user.created_via = app_name

        user.save()
        return user

    def _remove_password_fields(self):
        del self.fields['password']
        del self.fields['confirm_password']


class ForgotPasswordForm(forms.Form):
    email = forms.EmailField(
        label='Enter your email address'
    )


class ResetPasswordForm(forms.Form):
    user_id = forms.IntegerField(required=False, widget=forms.HiddenInput())
    token = forms.CharField(required=False, widget=forms.HiddenInput())
    password = forms.CharField(widget=forms.PasswordInput(), min_length=8)
    confirm_password = forms.CharField(widget=forms.PasswordInput())

    def clean(self):
        validate_passwords_match(self)
        return self.cleaned_data



from datetime import date
from unittest.mock import patch

from django import forms as django_forms
from django.conf import settings

from ... import forms


def test_validate_passwords_match_adds_error_to_form():
    class TestForm(django_forms.Form):
        password = django_forms.CharField(
            widget=django_forms.PasswordInput(),
            min_length=8
        )
        confirm_password = django_forms.CharField(
            widget=django_forms.PasswordInput()
        )

    form = TestForm()
    form.cleaned_data = {}
    form.cleaned_data['password'] = 'password'
    form.cleaned_data['confirm_password'] = 'password'

    forms.validate_passwords_match(form)
    assert not form.errors

    form.cleaned_data['confirm_password'] = 'something else'

    forms.validate_passwords_match(form)
    assert form.errors['confirm_password'][0] == 'Passwords do not match'


class TestSignUpForm:

    def test_create_returns_form(self):
        form = forms.SignUpForm.create()

        assert isinstance(form, forms.SignUpForm)
        assert not form.social_sign_in
        assert 'password' in form.fields
        assert 'confirm_password' in form.fields

    def test_create_socially_returns_form_without_password_fields(self):
        form = forms.SignUpForm.create(oauth_data={'x': 'y'})

        assert isinstance(form, forms.SignUpForm)
        assert form.social_sign_in
        assert 'password' not in form.fields
        assert 'confirm_password' not in form.fields

    @patch('kagiso_auth.forms.KagisoUser', autospec=True)
    def test_save_regular_sign_up(self, MockKagisoUser):  # noqa
        data = {
            'email': 'bogus@email.com',
            'first_name': 'Fred',
            'last_name': 'Smith',
            'password': 'mypassword',
            'confirm_password': 'mypassword',
            'mobile': '0821111111',
            'gender': 'MALE',
            'region': 'EASTERN_CAPE',
            'birth_date': date(1980, 1, 31),
            'alerts': ['EMAIL', 'SMS'],
        }
        form = forms.SignUpForm.create(post_data=data)

        assert form.is_valid()

        user = form.save(app_name=settings.APP_NAME)

        assert user.email == data['email']
        assert user.first_name == data['first_name']
        assert user.last_name == data['last_name']
        assert user.profile['mobile'] == data['mobile']
        assert user.profile['gender'] == data['gender']
        assert user.profile['region'] == data['region']
        assert user.profile['birth_date'] == str(data['birth_date'])
        assert user.profile['alerts'] == data['alerts']
        assert user.created_via == settings.APP_NAME

        user.set_password.assert_called_with(data['password'])

    @patch('kagiso_auth.forms.KagisoUser', autospec=True)
    def test_save_social_sign_up(self, MockKagisoUser):  # noqa
        data = {
            'provider': 'facebook',
            'email': 'bogus@email.com',
            'first_name': 'Fred',
            'last_name': 'Smith',
            'mobile': '0821111111',
            'gender': 'MALE',
            'region': 'EASTERN_CAPE',
            'birth_date': date(1980, 1, 31),
            'alerts': ['EMAIL', 'SMS'],
        }
        form = forms.SignUpForm.create(post_data=data, oauth_data=data)
        assert form.is_valid()

        user = form.save(app_name=settings.APP_NAME)

        assert user.email == data['email']
        assert user.email_confirmed
        assert user.first_name == data['first_name']
        assert user.last_name == data['last_name']
        assert user.profile['mobile'] == data['mobile']
        assert user.profile['gender'] == data['gender']
        assert user.profile['region'] == data['region']
        assert user.profile['birth_date'] == str(data['birth_date'])
        assert user.profile['alerts'] == data['alerts']
        assert user.created_via == settings.APP_NAME

        assert not user.set_password.called



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kagiso_auth', '0007_auto_20151130_1043'),
    ]

    operations = [
        migrations.AlterField(
            model_name='kagisouser',
            name='groups',
            field=models.ManyToManyField(related_query_name='user', help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', verbose_name='groups', related_name='user_set', blank=True, to='auth.Group'),
        ),
        migrations.AlterField(
            model_name='kagisouser',
            name='last_login',
            field=models.DateTimeField(null=True, blank=True, verbose_name='last login'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kagiso_auth', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='kagisouser',
            name='id',
            field=models.IntegerField(primary_key=True, serialize=False),
            preserve_default=True,
        ),
    ]
